/* $Id: cliopen.c,v 0.13 2002/01/19 19:10:22 sverrehu Exp $ */
/**************************************************************************
 *
 *  FILE            cliopen.c
 *  MODULE OF       shhnet - TCP/UDP networking library.
 *
 *  DESCRIPTION     A function for opening client side connections
 *                  for both tcp and udp.
 *                  Used internally.
 *
 *  WRITTEN BY      Sverre H. Huseby <shh@thathost.com>
 *
 **************************************************************************/

#include <stdlib.h>
#include <netinet/in.h>

#include "internal.h"
#include "shhnet.h"

/**************************************************************************
 *                                                                        *
 *                    P U B L I C    F U N C T I O N S                    *
 *                                                                        *
 **************************************************************************/

/*-------------------------------------------------------------------------
 *
 *  NAME          _snCliOpenCommon
 *
 *  FUNCTION      Open a tcp- or udp-connection.
 *
 *  INPUT         type    the connection type: SN_PROTO_TCP or SN_PROTO_UDP
 *                host    a hostname, or a hostaddress in dotted quad
 *                        notation.
 *                service a service name, or a port number (ascii). If a
 *                        service name is given, it is mapped to a port
 *                        number.
 *                udpconnect
 *                        boolean: should connect be done for udp?
 *                        connecting UDP makes it possible to use
 *                        read, write, send etc; functions that don't
 *                        take address and port parameters.
 *
 *  OUTPUT        sock    the socket referencing the connection.
 *
 *  RETURNS       Error code.
 *
 *  DESCRIPTION   This function will create a socket and do the following
 *                depending on whether to connection type is tcp or udp:
 *                tcp     1. Connect to the given port on the given host.
 *                udp     1. Bind to some local port
 *                        2. Possibly `connect', depending on the
 *                           udpconnect-variable.
 */
SNErr
_snCliOpenCommon(SNProto type, const char *host, const char *service,
		 int udpconnect, int *sock)
{
    SNErr  err;
    SNAddr servAddr;
    SNPort servPort;
    char   *servicename = NULL;

    switch (type) {
      case SN_PROTO_TCP:
	servicename = "tcp";
	break;
      case SN_PROTO_UDP:
	servicename = "udp";
	break;
    }
    
    /* get the port number corresponding to the given service. */
    if ((err = inServiceToPort(service, servicename, &servPort)) != SN_OK)
	return err;

    /* parse the server host address. */
    if ((err = inHostNameToAddr(host, &servAddr)) != SN_OK)
	return err;

    /* create a socket. */
    if ((err = sockCreate(type, sock)) != SN_OK)
	return err;

    /* bind the local end for udp, to have a valid return address.
     * TCP will do an implicite bind when connecting. */
    if (type == SN_PROTO_UDP)
	if ((err = sockBind(*sock, SN_ANY_ADDR, SN_ANY_PORT)) != SN_OK)
	    return err;

    /* always connect to the other end for TCP, and possibly connect for
     * UDP. the "connection" for UDP just stores the address of the server
     * in the OS, so that read, write etc. may be used without giving an
     * address. */
    if (type != SN_PROTO_UDP || udpconnect)
	if ((err = sockConnect(*sock, servAddr, servPort)) != SN_OK)
	    return err;

    return _snError(SN_OK, NULL);
}
