/* $Id: error.c,v 1.6 2002/01/19 19:10:22 sverrehu Exp $ */
/**************************************************************************
 *
 *  FILE            error.c
 *  MODULE OF       shhnet - TCP/UDP networking library.
 *
 *  DESCRIPTION     Functions for error handling.
 *
 *  WRITTEN BY      Sverre H. Huseby <shh@thathost.com>
 *
 **************************************************************************/

#include <string.h>
#include <errno.h>  /* errno error codes */
#include <netdb.h>  /* h_errno error codes */

#include "internal.h"
#include "shhnet.h"

extern int errno;
extern int h_errno;

/**************************************************************************
 *                                                                        *
 *                    P U B L I C    F U N C T I O N S                    *
 *                                                                        *
 **************************************************************************/

/*-------------------------------------------------------------------------
 *
 *  NAME          snSetErrorHandling
 *
 *  FUNCTION      Set error handling mode, and possibly a report function.
 *
 *  SYNOPSIS      #include "shhnet.h"
 *                int snSetErrorHandling(SNErrorHandling type,
 *                                       void (*f)(const char *, ...));
 *
 *  INPUT         type    the method used to handle any errors occurring
 *                        int the functions of this library:
 *
 *                          SN_REPORT_ERROR_AND_EXIT (default)
 *                            display an appropriate error message, and
 *                            abort the program.
 *                          SN_REPORT_ERROR_AND_RETURN
 *                            display an appropriate error message, and
 *                            return an error code from the function.
 *                          SN_EXIT
 *                            don't show any messages, just exit the
 *                            program.
 *                          SN_RETURN_ERROR
 *                            don't show any messages, just return an
 *                            error code.
 *
 *                f       function used to display error messages. the
 *                        function must accept printf-like arguments.
 *                        if NULL is given (the default), shhnet uses it's
 *                        own internal function.
 *                        this function is allowed to exit the program.
 *                        if it doesn't, and type is
 *                        SN_REPORT_ERROR_AND_RETURN, the library will
 *                        abort the program itself.
 *
 *  DESCRIPTION   Sets the way this library will deal with errors.
 */
void
snSetErrorHandling(SNErrorHandling type, void (*f)(const char *s, ...))
{
    _snErrorHandling = type;
    _snErrorPrintf = f;
}



/*-------------------------------------------------------------------------
 *
 *  NAME          snErrToStr
 *
 *  FUNCTION      Map an error code to a string.
 *
 *  SYNOPSIS      #include "shhnet.h"
 *                char *snErrToStr(SNErr err);
 *
 *  INPUT         err     the error code to get a string for.
 *
 *  RETURNS       Pointer to a string describing the error.
 *
 *  DESCRIPTION   Use this function to convert an error code to a
 *                descriptive string.
 *                Note that for this to work for error codes of types
 *                SN_ERR_ERRNO and SN_ERR_HERRNO, errno and h_errno must
 *                not have been changed explicitly or implicitly since
 *                the code that caused the error.
 */
const char *
snErrToStr(SNErr err)
{
    switch (err) {
      case SN_OK:
	return "";
      case SN_ERR_INTERNAL:
	return "internal error";
      case SN_ERR_ERRNO:
	return strerror(errno);
      case SN_ERR_H_ERRNO:
	switch (h_errno) {
	  case HOST_NOT_FOUND:
	    return "host not found";
	  case TRY_AGAIN:
	    return "name server busy. try again later";
	  case NO_RECOVERY:
	    return "host name lookup failure";
	  case NO_ADDRESS:
	    return "no network address for host";
	  default:
	    return "host name lookup failure";
	}
	break;
      case SN_ERR_UNKNOWN_SERVICE:
	return "unknown service";
    }
    return "some error";
}



/*-------------------------------------------------------------------------
 *
 *  NAME          snGetLastError
 *
 *  FUNCTION      Get code og last error.
 *
 *  SYNOPSIS      #include "shhnet.h"
 *                SNErr snGetLastError(void);
 *
 *  RETURNS       Last error code returned by the library.
 */
SNErr
snGetLastError(void)
{
    return _snLastError;
}
