/* $Id: midrform.c,v 1.4 2002/01/19 19:10:22 sverrehu Exp $ */
/**************************************************************************
 *
 *  FILE            midrform.c
 *  MODULE OF       shhnet - TCP/UDP networking library.
 *
 *  DESCRIPTION     Machine Independent Data Representation, routines for
 *                  filling and quering MIDR data using format strings.
 *                  The format strings contains a sequence of the following
 *                  codes:
 *
 *                      BYTE     B
 *                      SBYTE    b
 *                      WORD     W
 *                      SWORD    w
 *                      DWORD    D
 *                      SDWORD   d
 *
 *                      char *   s or sNUM
 *
 *                               NUM for strings is used only when fetching
 *                               strings. It represents the maximum
 *                               number of characters, including the
 *                               '\0'-byte (that is appended in case of
 *                               too many characters).
 *
 *  WRITTEN BY      Sverre H. Huseby <shh@thathost.com>
 *
 **************************************************************************/

#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <ctype.h>

#include "internal.h"
#include "shhmidr.h"

/**************************************************************************
 *                                                                        *
 *                    P U B L I C    F U N C T I O N S                    *
 *                                                                        *
 **************************************************************************/

/*-------------------------------------------------------------------------
 *
 *  NAME          midrPut
 *
 *  FUNCTION      Fill a MIDR-structure using format and variable arguments.
 *
 *  SYNOPSIS      #include "shhmidr.h"
 *                int midrPut(MIDR *midr, const char *format, ...);
 *
 *  INPUT         midr    the structure in question.
 *                format  a string specifying the types of data. see
 *                        description at header of this file.
 *                ...     data according to the format string.
 *
 *  RETURNS       The number of elements successfully stored.
 *
 *  DESCRIPTION   Extends the given MIDR-structure using printf-like
 *                format string, and variable arguments.
 */
int
midrPut(MIDR *midr, const char *format, ...)
{
    int     ret = 0;
    char    c, *s;
    va_list ap;

    va_start(ap, format);
    while ((c = *format++) != '\0') {
	switch(c) {
	  case 'B': /* BYTE */
	    if (midr->idx + midrSizeByte > midr->size)
		goto finish;
	    /* according to the C FAQ, arguments shorter that int are
	     * widened to int on the stack. */
	    midrPutByte(midr, (BYTE) va_arg(ap, unsigned));
	    break;
	  case 'b': /* SBYTE */
	    if (midr->idx + midrSizeSByte > midr->size)
		goto finish;
	    midrPutSByte(midr, (SBYTE) va_arg(ap, int));
	    break;
	  case 'W': /* WORD */
	    if (midr->idx + midrSizeWord > midr->size)
		goto finish;
	    midrPutWord(midr, (WORD) va_arg(ap, unsigned));
	    break;
	  case 'w': /* SWORD */
	    if (midr->idx + midrSizeSWord > midr->size)
		goto finish;
	    midrPutSWord(midr, (SWORD) va_arg(ap, int));
	    break;
	  case 'D': /* DWORD */
	    if (midr->idx + midrSizeDWord > midr->size)
		goto finish;
	    midrPutDWord(midr, va_arg(ap, DWORD));
	    break;
	  case 'd': /* SDWORD */
	    if (midr->idx + midrSizeSDWord > midr->size)
		goto finish;
	    midrPutSDWord(midr, va_arg(ap, SDWORD));
	    break;
	  case 's': /* char * */
	    s = va_arg(ap, char *);
	    if (midr->idx + strlen(s) + 1 > midr->size)
		goto finish;
	    midrPutStr(midr, s);
	    break;
	  default:
	    snSetErrorHandling(SN_REPORT_ERROR_AND_EXIT, NULL);
	    _snError(SN_ERR_INTERNAL, "midrPut");
	    /* no return from _snError in this case */
	}
	++ret;
    }
  finish:
    va_end(ap);
    return ret;
}



/*-------------------------------------------------------------------------
 *
 *  NAME          midrGet
 *
 *  FUNCTION      Read from a MIDR using format and variable arguments.
 *
 *  SYNOPSIS      #include "shhmidr.h"
 *                int midrGet(MIDR *midr, const char *format, ...);
 *
 *  INPUT         midr    the structure in question.
 *                format  a string specifying the types of data. see
 *                        description at header of this file.
 *
 *  OUTPUT        ...     pointers according to the format string.
 *
 *  RETURNS       The number of elements successfully fetched.
 *
 *  DESCRIPTION   Read from the given MIDR-structure using scanff-like
 *                format string, and variable arguments.
 */
int
midrGet(MIDR *midr, const char *format, ...)
{
    int     ret = 0, maxLen;
    char    c, *s;
    va_list ap;

    va_start(ap, format);
    while ((c = *format++) != '\0') {
	switch(c) {
	  case 'B': /* BYTE */
	    if (midr->idx + midrSizeByte > midr->len)
		goto finish;
	    *(va_arg(ap, BYTE *)) = midrGetByte(midr);
	    break;
	  case 'b': /* SBYTE */
	    if (midr->idx + midrSizeSByte > midr->len)
		goto finish;
	    *(va_arg(ap, SBYTE *)) = midrGetSByte(midr);
	    break;
	  case 'W': /* WORD */
	    if (midr->idx + midrSizeWord > midr->len)
		goto finish;
	    *(va_arg(ap, WORD *)) = midrGetWord(midr);
	    break;
	  case 'w': /* SWORD */
	    if (midr->idx + midrSizeSWord > midr->len)
		goto finish;
	    *(va_arg(ap, SWORD *)) = midrGetSWord(midr);
	    break;
	  case 'D': /* DWORD */
	    if (midr->idx + midrSizeDWord > midr->len)
		goto finish;
	    *(va_arg(ap, DWORD *)) = midrGetDWord(midr);
	    break;
	  case 'd': /* SDWORD */
	    if (midr->idx + midrSizeSDWord > midr->len)
		goto finish;
	    *(va_arg(ap, SDWORD *)) = midrGetSDWord(midr);
	    break;
	  case 's': /* char * */
	    if (isdigit(*format)) {
		s = va_arg(ap, char *);
		maxLen = 0;
		do {
		    maxLen = maxLen * 10 + (*format - '0');
		} while (isdigit(*++format));
		if (midr->idx + 1 > midr->len)
		    goto finish;
		midrGetStr(midr, s, maxLen);
		break;
	    }
	    /* fall through on missing digits */
	  default:
	    snSetErrorHandling(SN_REPORT_ERROR_AND_EXIT, NULL);
	    _snError(SN_ERR_INTERNAL, "midrGet");
	    /* no return from _snError in this case */
	}
	++ret;
    }
  finish:
    va_end(ap);
    return ret;
}
