/* $Id: midrsgn.c,v 1.5 2002/01/19 19:10:22 sverrehu Exp $ */
/**************************************************************************
 *
 *  FILE            midrsgn.c
 *  MODULE OF       shhnet - TCP/UDP networking library.
 *
 *  DESCRIPTION     Machine Independent Data Representation, routines
 *                  for handling signed entities.
 *
 *                  See midrgen.c for more info.
 *
 *  WRITTEN BY      Sverre H. Huseby <shh@thathost.com>
 *
 **************************************************************************/

#include "shhmidr.h"

/**************************************************************************
 *                                                                        *
 *                    P U B L I C    F U N C T I O N S                    *
 *                                                                        *
 **************************************************************************/

/*-------------------------------------------------------------------------
 *
 *  NAME          midrPutSByte
 *
 *  FUNCTION      Enter a signed byte in the buffer of the MIDR-structure.
 *
 *  SYNOPSIS      #include "shhmidr.h"
 *                void midrPutSByte(MIDR *midr, SBYTE x);
 *
 *  INPUT         midr    the structure in question.
 *                x       the signed byte to store.
 */
void
midrPutSByte(MIDR *midr, SBYTE x)
{
    SBYTE *p;

    p = (SBYTE *) (midr->buff + midr->idx);
    *p = x;
    if ((midr->idx += sizeof(SBYTE)) > midr->len)
	midr->len = midr->idx;
}



/*-------------------------------------------------------------------------
 *
 *  NAME          midrGetSByte
 *
 *  FUNCTION      Fetch a signed byte from the MIDR-structure.
 *
 *  SYNOPSIS      #include "shhmidr.h"
 *                SBYTE midrGetSByte(MIDR *midr);
 *
 *  INPUT         midr    the structure in question.
 *
 *  RETURNS       The next signed byte from the buffer.
 */
SBYTE
midrGetSByte(MIDR *midr)
{
    SBYTE *p;

    p = (SBYTE *) (midr->buff + midr->idx);
    midr->idx += sizeof(SBYTE);
    return *p;
}



/*-------------------------------------------------------------------------
 *
 *  NAME          midrPutSWord
 *
 *  FUNCTION      Enter a signed word in the buffer of the MIDR-structure.
 *
 *  SYNOPSIS      #include "shhmidr.h"
 *                void midrPutSWord(MIDR *midr, SWORD x);
 *
 *  INPUT         midr    the structure in question.
 *                x       the signed word to store.
 */
void
midrPutSWord(MIDR *midr, SWORD x)
{
    register int q;
    BYTE *p;

    p = (BYTE *) (midr->buff + midr->idx + sizeof(SWORD));
    for (q = sizeof(SWORD); q; q--) {
	*--p = x & 0xFF;
	x >>= 8;
    }
    if ((midr->idx += sizeof(SWORD)) > midr->len)
	midr->len = midr->idx;
}



/*-------------------------------------------------------------------------
 *
 *  NAME          midrGetSWord
 *
 *  FUNCTION      Fetch a signed word from the MIDR-structure.
 *
 *  SYNOPSIS      #include "shhmidr.h"
 *                SWORD midrGetSWord(MIDR *midr);
 *
 *  INPUT         midr    the structure in question.
 *
 *  RETURNS       The next signed word from the buffer.
 */
SWORD
midrGetSWord(MIDR *midr)
{
    register int q;
    BYTE *p;
    SWORD ret = 0;

    p = (BYTE *) (midr->buff + midr->idx);
    for (q = sizeof(SWORD); q; q--)
	ret = (ret << 8) | *p++;
    midr->idx += sizeof(SWORD);
    return ret;
}



/*-------------------------------------------------------------------------
 *
 *  NAME          midrPutSDWord
 *
 *  FUNCTION      Enter a signed dword in the buffer of the MIDR-structure.
 *
 *  SYNOPSIS      #include "shhmidr.h"
 *                void midrPutSDWord(MIDR *midr, SDWORD x);
 *
 *  INPUT         midr    the structure in question.
 *                x       the signed dword to store.
 */
void
midrPutSDWord(MIDR *midr, SDWORD x)
{
    register int q;
    BYTE *p;

    p = (BYTE *) (midr->buff + midr->idx + sizeof(SDWORD));
    for (q = sizeof(SDWORD); q; q--) {
	*--p = x & 0xFF;
	x >>= 8;
    }
    if ((midr->idx += sizeof(SDWORD)) > midr->len)
	midr->len = midr->idx;
}



/*-------------------------------------------------------------------------
 *
 *  NAME          midrGetSDWord
 *
 *  FUNCTION      Fetch a signed dword from the MIDR-structure.
 *
 *  SYNOPSIS      #include "shhmidr.h"
 *                SDWORD midrGetSDWord(MIDR *midr);
 *
 *  INPUT         midr    the structure in question.
 *
 *  RETURNS       The next signed dword from the buffer.
 */
SDWORD
midrGetSDWord(MIDR *midr)
{
    register int q;
    BYTE *p;
    SDWORD ret = 0;

    p = (BYTE *) (midr->buff + midr->idx);
    for (q = sizeof(SDWORD); q; q--)
	ret = (ret << 8) | *p++;
    midr->idx += sizeof(SDWORD);
    return ret;
}
