/* $Id: echo-server.c,v 1.3 2002/01/19 19:15:06 sverrehu Exp $ */
/**************************************************************************
 *
 *  FILE            echo-server.c
 *
 *  DESCRIPTION     Sample echo daemon, to illustrate the use of the
 *                  shhnet library for creating a TCP server.
 *
 *  WRITTEN BY      Sverre H. Huseby <shh@thathost.com>
 *
 **************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <unistd.h>

#include <shhnet.h>

#define DEBUG

#define PROG_NAME "echo-server"
#define ECHO_PORT "9001"

/**************************************************************************
 *                                                                        *
 *                   P R I V A T E    F U N C T I O N S                   *
 *                                                                        *
 **************************************************************************/

/*-------------------------------------------------------------------------
 *
 *  NAME          sigTermHandler
 *
 *  FUNCTION      Handle various program termination signals.
 *
 *  INPUT         s       the signal number.
 *
 *  DESCRIPTION   When the program is about to be killed, this function
 *                will do various cleaning.
 */
static void
sigTermHandler(int s)
{
    /* perform any cleanup necessary. */
#ifdef DEBUG
    fprintf(stderr, "%s: got signal %d, cleaning up and exiting\n",
	    PROG_NAME, s);
#endif
    exit(0);
}



/*-------------------------------------------------------------------------
 *
 *  NAME          sigHupHandler
 *
 *  FUNCTION      Handle SIGHUP signal.
 *
 *  INPUT         s       the signal number. not used.
 *
 *  DESCRIPTION   SIGHUP is typically sent to daemon processes to make
 *                them restart by reading their init-files.
 */
static void
sigHupHandler(int s)
{
    int olderrno;

    olderrno = errno;
    /* perform reinitializing. */
#ifdef DEBUG
    fprintf(stderr, "%s: got SIGHUP, resetting\n", PROG_NAME);
#endif

    errno = olderrno;
}



/*-------------------------------------------------------------------------
 *
 *  NAME          handleConnection
 *
 *  FUNCTION      Handle a connection from a remote host.
 *
 *  INPUT         sock    the socket referencing the open connection.
 *
 *  DESCRIPTION   This function does the communication with the other
 *                side. Setting up and closing down the connection is
 *                handeled by the caller.
 */
static void
handleConnection(int sock)
{
    char line[161];

    for (;;) {
	tcpGets(sock, line, sizeof(line));
	if (strlen(line) == 0)
	    break;
	tcpPuts(sock, line);
    }
}



/*-------------------------------------------------------------------------
 *
 *  NAME          waitForConnection
 *
 *  FUNCTION      Loop waiting for incoming connections.
 *
 *  DESCRIPTION   Waits for incoming connections. When any arrive, fork()'s
 *                to let a child handle the connection, and immediately
 *                waits for further connections.
 */
static void
waitForConnection(void)
{
    int sock, newsock, pid;

    tcpServerOpen(ECHO_PORT, &sock);
    for (;;) {
	tcpAccept(sock, &newsock);
	if ((pid = fork()) == 0) {
	    /* the child process handles this connection. */
#ifdef DEBUG
	    char peer[100];

	    inGetPeerStr(newsock, peer, 100, 1);
	    fprintf(stderr, "%s: connect from %s\n", PROG_NAME, peer);
#endif
	    tcpClose(sock);
	    sock = -1;
	    handleConnection(newsock);
	    tcpClose(newsock);
#ifdef DEBUG
	    fprintf(stderr, "%s: finished handling connection from %s\n",
		    PROG_NAME, peer);
#endif
	    exit(0);
	} else if (pid < 0) {
	    /* couldn't fork for some reason. */
#ifdef DEBUG
	    fprintf(stderr, "%s: oops! unable to fork!\n", PROG_NAME);
#endif
	}
	tcpClose(newsock);
    }
}



/**************************************************************************
 *                                                                        *
 *                    P U B L I C    F U N C T I O N S                    *
 *                                                                        *
 **************************************************************************/

int
main(int argc, char *argv[])
{
    /* fork off a child process to enable the program to return immediately. */
    if (fork() != 0) {
	/* parent just exits. */
	return 0;
    }
    snSetErrorHandling(SN_REPORT_ERROR_AND_EXIT, NULL);
    snInit();
    snServerSetupSigHandlers(sigTermHandler, sigHupHandler);

#ifdef DEBUG
    fprintf(stderr, "%s started\n", PROG_NAME);
#endif
    waitForConnection();
#ifdef DEBUG
    fprintf(stderr, "%s finished\n", PROG_NAME);
#endif

    snFinish();
    return 0;
}
