/* $Id: socket.c,v 1.8 2002/01/19 19:10:22 sverrehu Exp $ */
/**************************************************************************
 *
 *  FILE            socket.c
 *  MODULE OF       shhnet - TCP/UDP networking library.
 *
 *  DESCRIPTION     Functions directly connected to socket, such as setting
 *                  socket options.
 *
 *  WRITTEN BY      Sverre H. Huseby <shh@thathost.com>
 *
 **************************************************************************/

#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netinet/tcp.h> /* Irix wanted this one for TCP_NODELAY */

#include "internal.h"
#include "shhnet.h"

/**************************************************************************
 *                                                                        *
 *                    P U B L I C    F U N C T I O N S                    *
 *                                                                        *
 **************************************************************************/

/*-------------------------------------------------------------------------
 *
 *  NAME          sockCreate
 *
 *  FUNCTION      Create a socket of specified type.
 *
 *  SYNOPSIS      #include "shhnet.h"
 *                SNErr sockCreate(SNProto proto);
 *
 *  INPUT         proto   the protocol type: SN_PROTO_TCP or SN_PROTO_UDP
 *
 *  OUTPUT        sock    the socket.
 *
 *  RETURNS       Error code.
 */
SNErr
sockCreate(SNProto proto, int *sock)
{
    int socktype = 0;

    switch (proto) {
      case SN_PROTO_TCP:
	socktype = SOCK_STREAM;
	break;
      case SN_PROTO_UDP:
	socktype = SOCK_DGRAM;
	break;
    }
    
    /* create a socket. */
    if ((*sock = socket(AF_INET, socktype, 0)) < 0)
	return _snError(SN_ERR_ERRNO, "socket");

    return _snError(SN_OK, NULL);
}



/*-------------------------------------------------------------------------
 *
 *  NAME          sockBind
 *
 *  FUNCTION      Bind local end of an association.
 *
 *  SYNOPSIS      #include "shhnet.h"
 *                SNErr sockBind(int sock, SNPort port);
 *
 *  INPUT         sock    the socket to bind.
 *                addr    the local address to bind to, network byte order.
 *                        use SN_ANY_ADDR to accept incoming packets on
 *                        all interfaces on a multihomed host.
 *                port    the local port to bind to, network byte order.
 *
 *  RETURNS       Error code.
 *
 *  DESCRIPTION   Binding fills in the local address and port of an
 *                association 5-tuple.
 */
SNErr
sockBind(int sock, SNAddr addr, SNPort port)
{
    struct sockaddr_in sa;

    memset(&sa, 0, sizeof(sa));
    sa.sin_family = AF_INET;
    sa.sin_addr.s_addr = addr;
    sa.sin_port = port;

    if (bind(sock, (struct sockaddr *) &sa, sizeof(sa)) < 0)
	return _snError(SN_ERR_ERRNO, "bind");
    return _snError(SN_OK, NULL);
}



/*-------------------------------------------------------------------------
 *
 *  NAME          sockConnect
 *
 *  FUNCTION      Connect the foreign end of an association.
 *
 *  SYNOPSIS      #include "shhnet.h"
 *                SNErr sockConnect(int sock, SNAddr addr, SNPort port);
 *
 *  INPUT         sock    the socket to connect.
 *                addr    the address to connect to, network byte order.
 *                port    the port to connect to, network byte order.
 *
 *  RETURNS       Error code.
 *
 *  DESCRIPTION   Connecting a socket means different things for TCP and
 *                UDP, as UDP has no means of a connection:
 *
 *                TCP     Attempts to establish a connection to a socket
 *                        on a (possibly) remote host.
 *                UDP     Sets the default source and destination for
 *                        sending and receiving datagrams. For UDP,
 *                        sockConnect may be called multiple times.
 */
SNErr
sockConnect(int sock, SNAddr addr, SNPort port)
{
    struct sockaddr_in sa;

    memset(&sa, 0, sizeof(sa));
    sa.sin_family = AF_INET;
    sa.sin_addr.s_addr = addr;
    sa.sin_port = port;

    if (connect(sock, (struct sockaddr *) &sa, sizeof(sa)) < 0)
	return _snError(SN_ERR_ERRNO, "connect");
    return _snError(SN_OK, NULL);
}



/*-------------------------------------------------------------------------
 *
 *  NAME          sockSetNoDelay
 *
 *  FUNCTION      Specify whether to buffer up several small TCP-packets.
 *
 *  SYNOPSIS      #include "shhnet.h"
 *                SNErr sockSetNoDelay(int sock, int onoff);
 *
 *  INPUT         sock    the socket to modify.
 *                onoff   a boolean. 1=on, 0=off.
 *
 *  RETURNS       Error code.
 *
 *  DESCRIPTION   When disabled, bytes are sent over a TCP-stream as soon
 *                as they are output. Suitable (and required) for
 *                interactive programs that want to send a single keypress
 *                at a time.
 *                Usable for TCP (SOCK_STREAM) only.
 */
SNErr
sockSetNoDelay(int sock, int onoff)
{
    if (setsockopt(sock, IPPROTO_TCP, TCP_NODELAY,
		   (char *) &onoff, sizeof(onoff)) < 0)
	return _snError(SN_ERR_ERRNO, "setsockopt");
    return _snError(SN_OK, NULL);
}



/*-------------------------------------------------------------------------
 *
 *  NAME          sockSetBroadcast
 *
 *  FUNCTION      Specify whether to allow broadcasting on a socket.
 *
 *  SYNOPSIS      #include "shhnet.h"
 *                SNErr sockSetBroadcast(int sock, int onoff);
 *
 *  INPUT         sock    the socket to modify.
 *                onoff   a boolean. 1=on, 0=off.
 *
 *  RETURNS       Error code.
 *
 *  DESCRIPTION   Usable for UDP (SOCK_DGRAM) only.
 */
SNErr
sockSetBroadcast(int sock, int onoff)
{
    if (setsockopt(sock, SOL_SOCKET, SO_BROADCAST,
		   (char *) &onoff, sizeof(onoff)) < 0)
	return _snError(SN_ERR_ERRNO, "setsockopt");
    return _snError(SN_OK, NULL);
}



/*-------------------------------------------------------------------------
 *
 *  NAME          sockSetKeepAlive
 *
 *  FUNCTION      Specify whether to preiodically check the connection.
 *
 *  SYNOPSIS      #include "shhnet.h"
 *                SNErr sockSetKeepAlive(int sock, int onoff);
 *
 *  INPUT         sock    the socket to modify.
 *                onoff   a boolean. 1=on, 0=off.
 *
 *  RETURNS       Error code.
 *
 *  DESCRIPTION   When enabled, data is transmitted periodically when no
 *                other data is normally sent, to be able to detect a
 *                broken connection.
 *                Usable for TCP (SOCK_STREAM) only.
 */
SNErr
sockSetKeepAlive(int sock, int onoff)
{
    if (setsockopt(sock, SOL_SOCKET, SO_KEEPALIVE,
		   (char *) &onoff, sizeof(onoff)) < 0)
	return _snError(SN_ERR_ERRNO, "setsockopt");
    return _snError(SN_OK, NULL);
}



/*-------------------------------------------------------------------------
 *
 *  NAME          sockSetLinger
 *
 *  FUNCTION      Specify whether to delay closing a socket to transfer data.
 *
 *  SYNOPSIS      #include "shhnet.h"
 *                SNErr sockSetLinger(int sock, int onoff);
 *
 *  INPUT         sock    the socket to modify.
 *                onoff   a boolean. 1=on, 0=off.
 *
 *  RETURNS       Error code.
 *
 *  DESCRIPTION   When lingering is on, any close on the socket will wait
 *                untill all data is transfered.
 *                Usable for TCP (SOCK_STREAM) only.
 */
SNErr
sockSetLinger(int sock, int onoff)
{
    struct linger li;

    li.l_onoff = onoff;
    li.l_linger = 5;  /* #seconds. not used, according to Stevens. */
    if (setsockopt(sock, SOL_SOCKET, SO_LINGER, (char *) &li, sizeof(li)) < 0)
	return _snError(SN_ERR_ERRNO, "setsockopt");
    return _snError(SN_OK, NULL);
}



/*-------------------------------------------------------------------------
 *
 *  NAME          sockSetOutOfBandInline
 *
 *  FUNCTION      Specify whether to inline out of band data.
 *
 *  SYNOPSIS      #include "shhnet.h"
 *                SNErr sockSetOutOfBandInline(int sock, int onoff);
 *
 *  INPUT         sock    the socket to modify.
 *                onoff   a boolean. 1=on, 0=off.
 *
 *  RETURNS       Error code.
 *
 *  DESCRIPTION   When enabled, out of band data is duplicated in the
 *                normal input queue.
 *                Usable for TCP (SOCK_STREAM) only.
 */
SNErr
sockSetOutOfBandInline(int sock, int onoff)
{
    if (setsockopt(sock, SOL_SOCKET, SO_OOBINLINE,
		   (char *) &onoff, sizeof(onoff)) < 0)
	return _snError(SN_ERR_ERRNO, "setsockopt");
    return _snError(SN_OK, NULL);
}



/*-------------------------------------------------------------------------
 *
 *  NAME          sockSetReuseAddr
 *
 *  FUNCTION      Allow local part of association tuples to be reused.
 *
 *  SYNOPSIS      #include "shhnet.h"
 *                SNErr sockSetReuseAddr(int sock, int onoff);
 *
 *  INPUT         sock    the socket to modify.
 *                onoff   a boolean. 1=on, 0=off.
 *
 *  RETURNS       Error code.
 *
 *  DESCRIPTION   When enabled, this enables the local address and port pair
 *                of an association to be reused.
 *                Usable for TCP (SOCK_STREAM) and UDP (SOCK_DGRAM).
 */
SNErr
sockSetReuseAddr(int sock, int onoff)
{
    if (setsockopt(sock, SOL_SOCKET, SO_REUSEADDR,
		   (char *) &onoff, sizeof(onoff)) < 0)
	return _snError(SN_ERR_ERRNO, "setsockopt");
#ifdef SO_REUSEPORT
    if (setsockopt(sock, SOL_SOCKET, SO_REUSEPORT,
		   (char *) &onoff, sizeof(onoff)) < 0)
	return _snError(SN_ERR_ERRNO, "setsockopt");
#endif
    return _snError(SN_OK, NULL);
}



/*-------------------------------------------------------------------------
 *
 *  NAME          sockJoinMulticastGroup
 *
 *  FUNCTION      Become a member of a multicast group.
 *
 *  SYNOPSIS      #include "shhnet.h"
 *                SNErr sockJoinMulticastGroup(int sock, SNAddr group);
 *
 *  INPUT         sock    the socket to modify.
 *                group   address of group to join, network byte order.
 *
 *  RETURNS       Error code.
 *
 *  DESCRIPTION   In order to receive from a multicast group, a process
 *                must join the group. This function adds group membership
 *                on the default multicastcapable interface.
 *                A socket may be member of several groups.
 *                Usable for UDP (SOCK_DGRAM) and IP (SOCK_RAW).
 */
SNErr
sockJoinMulticastGroup(int sock, SNAddr group)
{
    struct ip_mreq mreq;

    mreq.imr_multiaddr.s_addr = group;
    mreq.imr_interface.s_addr = SN_ANY_ADDR;
    if (setsockopt(sock, IPPROTO_IP, IP_ADD_MEMBERSHIP,
		   (char *) &mreq, sizeof(mreq)) < 0)
	return _snError(SN_ERR_ERRNO, "setsockopt");
    return _snError(SN_OK, NULL);
}



/*-------------------------------------------------------------------------
 *
 *  NAME          sockLeaveMulticastGroup
 *
 *  FUNCTION      Drop membership from a multicast group.
 *
 *  SYNOPSIS      #include "shhnet.h"
 *                SNErr sockLeaveMulticastGroup(int sock, SNAddr group);
 *
 *  INPUT         sock    the socket to modify.
 *                group   address of group to leave, network byte order.
 *
 *  RETURNS       Error code.
 *
 *  DESCRIPTION   Leaves a group previously joined by
 *                sockJoinMulticastGroup. The membership is also dropped
 *                automatically when the socket is closed.
 *                Usable for UDP (SOCK_DGRAM) and IP (SOCK_RAW).
 */
SNErr
sockLeaveMulticastGroup(int sock, SNAddr group)
{
    struct ip_mreq mreq;

    mreq.imr_multiaddr.s_addr = group;
    mreq.imr_interface.s_addr = SN_ANY_ADDR;
    if (setsockopt(sock, IPPROTO_IP, IP_DROP_MEMBERSHIP,
		   (char *) &mreq, sizeof(mreq)) < 0)
	return _snError(SN_ERR_ERRNO, "setsockopt");
    return _snError(SN_OK, NULL);
}



/*-------------------------------------------------------------------------
 *
 *  NAME          sockSetMulticastTtl
 *
 *  FUNCTION      Drop membership from a multicast group.
 *
 *  SYNOPSIS      #include "shhnet.h"
 *                SNErr sockSetMulticastTtl(int sock, int ttl);
 *
 *  INPUT         sock    the socket to modify.
 *                ttl     time to live -value [0..255].
 *                          0 same host
 *                          1 same subnet
 *                         12 same site
 *                         64 same region
 *                        128 same continent
 *                        255 unrestricted
 *
 *  RETURNS       Error code.
 *
 *  DESCRIPTION   Leaves a group previously joined by
 *                sockJoinMulticastGroup. The membership is also dropped
 *                automatically when the socket is closed.
 *                Usable for UDP (SOCK_DGRAM) and IP (SOCK_RAW).
 */
SNErr
sockSetMulticastTtl(int sock, int ttl)
{
    unsigned char theTtl;

    theTtl = ttl > 255 ? 255 : ttl;
    if (setsockopt(sock, IPPROTO_IP, IP_MULTICAST_TTL,
		   (char *) &theTtl, sizeof(theTtl)) < 0)
	return _snError(SN_ERR_ERRNO, "setsockopt");
    return _snError(SN_OK, NULL);
}



/*-------------------------------------------------------------------------
 *
 *  NAME          sockSetMulticastLoop
 *
 *  FUNCTION      Set whether multicast packets should be looped back.
 *
 *  SYNOPSIS      #include "shhnet.h"
 *                SNErr sockSetMulticastLoop(int sock, int onoff);
 *
 *  INPUT         sock    the socket to modify.
 *                onoff   a boolean. 1=on, 0=off.
 *
 *  RETURNS       Error code.
 *
 *  DESCRIPTION   When enabled, this allows multicast packets to be looped
 *                back on the sending host, so that processes on this host
 *                will receive the packets. The default is to enable looping.
 *                Usable for UDP (SOCK_DGRAM) and IP (SOCK_RAW).
 */
SNErr
sockSetMulticastLoop(int sock, int onoff)
{
    unsigned char status;

    status = onoff;
    if (setsockopt(sock, IPPROTO_IP, IP_MULTICAST_LOOP,
		   (char *) &status, sizeof(status)) < 0)
	return _snError(SN_ERR_ERRNO, "setsockopt");
    return _snError(SN_OK, NULL);
}
